/*
 * QEMU Confidential Guest support
 *
 * Copyright Red Hat.
 *
 * Authors:
 *  David Gibson <david@gibson.dropbear.id.au>
 *
 * This work is licensed under the terms of the GNU GPL, version 2 or
 * later.  See the COPYING file in the top-level directory.
 *
 */

#include "qemu/osdep.h"

#include "exec/confidential-guest-support.h"
#include "qemu/error-report.h"
#include "qapi/error.h"
#include "exec/igvm.h"

OBJECT_DEFINE_ABSTRACT_TYPE(ConfidentialGuestSupport,
                            confidential_guest_support,
                            CONFIDENTIAL_GUEST_SUPPORT,
                            OBJECT)

#if defined(CONFIG_IGVM)
static char *get_igvm(Object *obj, Error **errp)
{
    ConfidentialGuestSupport *cgs = CONFIDENTIAL_GUEST_SUPPORT(obj);
    return g_strdup(cgs->igvm_filename);
}

static void set_igvm(Object *obj, const char *value, Error **errp)
{
    ConfidentialGuestSupport *cgs = CONFIDENTIAL_GUEST_SUPPORT(obj);
    g_free(cgs->igvm_filename);
    cgs->igvm_filename = g_strdup(value);
#if defined(CONFIG_IGVM)
    igvm_file_init(cgs, errp);
#endif
}
#endif

static void confidential_guest_support_class_init(ObjectClass *oc, void *data)
{
#if defined(CONFIG_IGVM)
    object_class_property_add_str(oc, "igvm-file",
        get_igvm, set_igvm);
    object_class_property_set_description(oc, "igvm-file",
        "Set the IGVM filename to use");
#endif
}

static int check_support(ConfidentialGuestPlatformType platform,
                         uint16_t platform_version, uint8_t highest_vtl,
                         uint64_t shared_gpa_boundary)
{
    /* Default: no support. */
    return 0;
}

static int set_guest_state(hwaddr gpa, uint8_t *ptr, uint64_t len,
                           ConfidentialGuestPageType memory_type,
                           uint16_t cpu_index, Error **errp)
{
    error_setg(errp,
               "Setting confidential guest state is not supported for this platform");
    return -1;
}

static int set_guest_policy(ConfidentialGuestPolicyType policy_type,
                            uint64_t policy,
                            void *policy_data1, uint32_t policy_data1_size,
                            void *policy_data2, uint32_t policy_data2_size,
                            Error **errp)
{
    error_setg(errp,
               "Setting confidential guest policy is not supported for this platform");
    return -1;
}

static int get_mem_map_entry(int index, ConfidentialGuestMemoryMapEntry *entry,
                             Error **errp)
{
    error_setg(
        errp,
        "Obtaining the confidential guest memory map is not supported for this platform");
    return -1;
}

static void confidential_guest_support_init(Object *obj)
{
    ConfidentialGuestSupport *cgs = CONFIDENTIAL_GUEST_SUPPORT(obj);
    cgs->check_support = check_support;
    cgs->set_guest_state = set_guest_state;
    cgs->set_guest_policy = set_guest_policy;
    cgs->get_mem_map_entry = get_mem_map_entry;
}

static void confidential_guest_support_finalize(Object *obj)
{
}

bool cgs_is_igvm(ConfidentialGuestSupport *cgs)
{
#if defined(CONFIG_IGVM)
    return cgs && cgs->igvm;
#else
    return false;
#endif
}

void cgs_process_igvm(ConfidentialGuestSupport *cgs)
{
#if defined(CONFIG_IGVM)
    if (cgs && cgs_is_igvm(cgs)) {
        igvm_process(cgs, &error_fatal);
    }
#endif
}
